Package: seqwrap
Type: Package
Title: Item-by-Item Iterative Model Fitting
Version: 0.7.0
Authors@R: c(
    person("Daniel", "Hammarstr\u00f6m",
    email = "daniel.hammarstrom@inn.no",
    role = c("aut", "cre", "cph"),
    comment = c(ORCID = "0000-0001-8360-2100")),
    person("Chidimma", "Echebiri",
    email = "chidimma.echebiri@inn.no",
    role = c("ctb"),
    comment = c(ORCID = "0000-0003-3134-5186")))
Description: Models high-dimensional data, such as RNA-seq or proteomic data
    using an item-by-item strategy. The package contains functions to wrap
    high-dimensional data and iterate over them using established R packages
    for regression modelling (e.g., 'glmmTMB' or 'mgcv').
License: GPL-3
Encoding: UTF-8
LazyData: true
LazyDataCompression: xz
URL: https://github.com/trainome/seqwrap
BugReports: https://github.com/trainome/seqwrap/issues
Depends: R (>= 4.1)
Imports: cli, S7, tibble, pbapply, parallel, stats, DHARMa, broom.mixed
Suggests: testthat (>= 3.0.0), dplyr, knitr, purrr, quarto, glmmTMB,
        lme4, nlme, MASS, rmarkdown, gt, edgeR, tidyselect, ggplot2,
        cowplot, ggtext, R.rsp
Config/testthat/edition: 3
Collate: 'seqwrap-global.R' 'aaa-.R' 'data-helper.R' 'data.R'
        'generic-summaries.R' 'seqwrap.R' 'seqwrap_mtf.R' 'simcounts.R'
        'simcounts2.R'
VignetteBuilder: quarto, R.rsp
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-15 14:17:25 UTC; 706194
Author: Daniel Hammarström [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-8360-2100>),
  Chidimma Echebiri [ctb] (ORCID:
    <https://orcid.org/0000-0003-3134-5186>)
Maintainer: Daniel Hammarström <daniel.hammarstrom@inn.no>
Repository: CRAN
Date/Publication: 2026-05-19 10:00:02 UTC
Built: R 4.6.0; ; 2026-05-19 11:20:24 UTC; unix
