# Generated by roxygen2: do not edit by hand

export(generic_evaluation)
export(generic_summary)
export(seqwrap)
export(seqwrapResults)
export(seqwrap_compose)
export(seqwrap_summarise)
export(simcounts)
export(simcounts2)
export(swcontainer)
importFrom(stats,as.formula)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,predict)
importFrom(stats,rnbinom)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,sigma)
